/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: FrontEngine.java,v 1.1 2007/06/11 13:34:07 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.engine.rmi.server;

import java.net.MalformedURLException;
import java.rmi.AlreadyBoundException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;
import javax.xml.namespace.QName;

import org.objectweb.petals.component.framework.ComponentPropertiesManager;
import org.objectweb.petals.component.framework.util.WSDLHelper;
import org.objectweb.petals.engine.rmi.server.registry.Registry;
import org.objectweb.petals.engine.rmi.server.registry.RmiProperties;
import org.objectweb.petals.tools.rmi.server.remote.implementations.RemoteComponentContextImpl;
import org.objectweb.petals.tools.rmi.server.remote.implementations.RemoteDeliveryChannelImpl;
import org.objectweb.petals.tools.rmi.server.remote.interfaces.RemoteComponentContext;
import org.objectweb.petals.tools.rmi.server.remote.interfaces.RemoteDeliveryChannel;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;

/**
 * Class FrontEngine.java.
 * @author Nicolas Salatge
 */
public class RmiComponent implements Component, ComponentLifeCycle {


	/**
	 * the RMI registry Server.
	 */
	private Registry rmiServer;

	/**
	 * the rmi component context.
	 */
	private RemoteComponentContext rmiComponentContext = null;

	/**
	 * the concrete component context.
	 */
	private javax.jbi.component.ComponentContext concreteComponentContext = null;

	/**
	 * the rmi delivery channel.
	 */
	private RemoteDeliveryChannel rmiDeliveryChannel = null;

	/**
	 * the concrete delivery channel.
	 */
	private javax.jbi.messaging.DeliveryChannel concreteDeliveryChannel = null;

	/**
	 * the component logger.
	 */
	private Logger logger;

	/**
	 * HasMap containing all the messages exchanges.
	 */
	private Map<String, MessageExchange> messages;

	/**
	 * The Rmi properties.
	 */
	private RmiProperties rmiProperties = null;

	/**
	 * Default Constructor.
	 */
	public RmiComponent() {
	}

	/**
	 * get the component logger.
	 * @return the component logger
	 */
	public Logger getLogger() {
		return this.logger;
	}

	/**
	 * get messaging.
	 * @return
	 *    the messages
	 */
	public Map<String, MessageExchange> getMessages() {
		return this.messages;
	}

	/**
	 * start the component.
	 * @throws JBIException
	 *             impossible to start the component.
	 */
	public void start() throws JBIException {

		this.getLogger().info("Start RMI engine :" + System.currentTimeMillis());

		if (this.getLogger().isLoggable(Level.INFO)) {
			this.getLogger().info("RMI Port used = " + this.rmiProperties.getRmiPort());
			this.getLogger().info("RMI Registry Server active = "
					+ String.valueOf(this.rmiProperties.isEmbeddedRegistry()));
			this.getLogger().info("RMI Component Context Name "
					+ "registered = " + this.rmiProperties.getComponentContextName());
		}


		System.setProperty("com.sun.xml.namespace."
				+ "QName.useCompatibleSerialVersionUID", "1.0");


		try {

			this.messages = new HashMap<String, MessageExchange>();

			this.rmiDeliveryChannel = new RemoteDeliveryChannelImpl(this.getChannel(),
					this.getContext(), this.getMessages(), this.getLogger());

			this.rmiComponentContext = new RemoteComponentContextImpl(this.getContext(), this.getRmiDeliveryChannel());

			this.rmiServer = new Registry(this.rmiProperties);

			this.rmiServer.initializeRegistry();

			this.rmiServer.registerRMIInterface(this.rmiComponentContext);

			this.getLogger().info("RMI engine listenning");

		} catch (RemoteException e) {
			String msg = "RemoteException: Can't create RMI interface";
			this.getLogger().severe(msg);
			throw new JBIException(msg, e);
		} catch (MalformedURLException e) {
			String msg = "MalformedURLException: Can't create RMI interface";
			this.getLogger().severe(msg);
			throw new JBIException(msg, e);
		} catch (AlreadyBoundException e) {
			String msg = "AlreadyBoundException: Can't create RMI interface";
			this.getLogger().severe(msg);
			throw new JBIException(msg, e);
		}

	}

	/**
	 * get the rmi component context.
	 * @return the rmi component context
	 */
	public RemoteComponentContext getRmiComponentContext() {
		return this.rmiComponentContext;
	}

	/**
	 * get the rmi delivery channel.
	 * @return the rmi delivery channel
	 */
	public RemoteDeliveryChannel getRmiDeliveryChannel() {
		return this.rmiDeliveryChannel;
	}

	/**
	 * stop the component.
	 * @throws JBIException
	 *             impossible to start the component.
	 */
	public void stop() throws JBIException {
		this.getLogger().info("Stop Front engine");

		try {
			this.rmiServer.deregisterRMIInterface();
		} catch (RemoteException e) {
			String msg = "RemoteException: Can't unbind RMI interface";
			this.getLogger().severe(msg);
			throw new JBIException(msg, e);
		} catch (MalformedURLException e) {
			String msg = "MalformedURLException: Can't unbind RMI interface";
			this.getLogger().severe(msg);
			throw new JBIException(msg, e);
		} catch (NotBoundException e) {
			String msg = "NotBoundException: Can't unbind RMI interface";
			this.getLogger().severe(msg);
			throw new JBIException(msg, e);
		}
	}

	/**
	 * Get the description of service.
	 * @param endpoint
	 *            the endpoint
	 * @return
	 *        the description
	 */
	public Document getServiceDescription(final ServiceEndpoint endpoint) {
		return WSDLHelper.createLightWSDL20(new QName(
				"http://petals.objectweb.org/",
				"petals-se-rmi-interface" + endpoint.getEndpointName()),
				endpoint.getServiceName(),
				endpoint.getEndpointName());
	}

	/**
	 * Get the component life cycle.
	 * @return the component life cylce
	 */
	public ComponentLifeCycle getLifeCycle() {
		return this;
	}

	/**
	 * Get the service unit manager.
	 * @return the service unit manager
	 */
	public ServiceUnitManager getServiceUnitManager() {
		return null;
	}

	/**
	 * Test if exhange with consumer is ok.
	 * @param endpoint
	 *              the endpoint
	 * @param exchange
	 *              the exchange
	 * @return true if ok
	 */
	public boolean isExchangeWithConsumerOkay(
			final ServiceEndpoint endpoint,
			final MessageExchange exchange) {
		return false;
	}

	/**
	 * Test if exhange with provider is ok.
	 * @param endpoint
	 *              the endpoint
	 * @param exchange
	 *              the exchange
	 * @return true if ok
	 */
	public boolean isExchangeWithProviderOkay(
			final ServiceEndpoint endpoint,
			final MessageExchange exchange) {
		return true;
	}

	/**
	 * Resolve the endpoint reference.
	 * @param epr
	 *         the document fragment
	 * @return the service endpoint
	 */
	public ServiceEndpoint resolveEndpointReference(
			final DocumentFragment epr) {
		return null;
	}

	/**
	 * Get the extension MBean name.
	 * @return the extension Mbean name
	 */
	public ObjectName getExtensionMBeanName() {
		return null;
	}

	/**
	 * init the component.
	 * @param context
	 *          the context
	 * @throws JBIException
	 *               impossible to init the component
	 */
	public void init(
			final javax.jbi.component.ComponentContext context)
	throws JBIException {

		this.concreteComponentContext = context;
		this.concreteDeliveryChannel = this.concreteComponentContext.getDeliveryChannel();
		this.logger = this.concreteComponentContext.getLogger("", null);
		this.rmiProperties = new RmiProperties(this.loadProperties());

		if (this.logger == null) {
			throw new JBIException("The logger is null");
		}

		this.logger.log(Level.INFO, "");

	}

	/**
	 * get the context.
	 * @return the component context
	 */
	public javax.jbi.component.ComponentContext getContext() {
		return this.concreteComponentContext;
	}

	/**
	 * get the delivery channel.
	 * @return the delivery channel
	 */
	public javax.jbi.messaging.DeliveryChannel getChannel() {
		return this.concreteDeliveryChannel;
	}

	/**
	 * get the shutdown.
	 * @throws JBIException
	 *               impossible to realize the shutdown
	 */
	public void shutDown() throws JBIException {
		this.logger.log(Level.INFO, "shutDown");
	}


	/*
	 * Added Method
	 */

	/**
	 * Load the extensions properties.
	 * @return
	 *     the component properties manager
	 */
	private ComponentPropertiesManager loadProperties() {
		ComponentPropertiesManager extensionProp = new ComponentPropertiesManager();
		extensionProp.setRootPath(this.getContext().getInstallRoot());
		extensionProp.load();
		return extensionProp;
	}

}
