/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.engine.rmi.server.registry;

import java.net.MalformedURLException;
import java.rmi.AlreadyBoundException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;

import org.objectweb.petals.tools.rmi.server.remote.interfaces.RemoteComponentContext;

/**
 * A class to simplify the rmi register creation.
 *
 * @author Nicolas Salatge
 */
public class Registry {

    /**
     * The registry itself.
     */
    private static java.rmi.registry.Registry registry;

    /**
     * A registry has been created ?
     */
    private static boolean registryCreator = false;

    /**
     * Rmi component context name.
     */
    private String rmiComponentContextName = null;

    /**
     * The registry port number.
     */
    private int registryPortNumber = -1;

    /**
     * Do we need to create a new registry.
     */
    private boolean shouldCreateRegistry = true;

    /**
     * Have we already checked the register status.
     */
    private boolean registryChecked;



    /**
     * Creates a new instance of Registry.
     *
     * @param properties
     *         the rmi properties
     */
    public Registry(final RmiProperties properties) {
        this.setRegistryPortNumber(properties.getRmiPort());

        this.setShouldCreateRegistry(properties.isEmbeddedRegistry());

        this.setRmiComponentContextName(properties.getComponentContextName());
    }

    /**
     * Get the registry port number.
     *
     * @return
     *       the port
     */
    public int getRegistryPortNumber() {
        return this.registryPortNumber;
    }

    /**
     * Set the registry port number.
     *
     * @param v
     *   the port
     */
    public void setRegistryPortNumber(final int v) {
        this.registryPortNumber = v;
        this.registryChecked = false;
    }

    /**
     * Do we need to create a new registry ?
     *
     * @return
     *    true if the server registry have to create
     */
    public boolean shouldCreateRegistry() {
        return this.shouldCreateRegistry;
    }

    /**
     * Set ShouldCreateRegistry variable.
     * @param v
     *     the ShouldCreateRegistry
     */
    public void setShouldCreateRegistry(final boolean v) {
        this.shouldCreateRegistry = v;
    }

    /**
     * Get the rmi component context name.
     * @return
     *       the rmi component context name
     */
    public String getRmiComponentContextName() {
        return this.rmiComponentContextName;
    }

    /**
     *
     * @param n
     *      the name of component context name
     */
    public void setRmiComponentContextName(final String n) {
        this.rmiComponentContextName = n;
    }

    /**
     * Initialize the registry. IF the registry already exists, get the
     * references on it, else launch it...
     * @throws RemoteException
     *             Impossible to realize a remote access
     *
     */
    public synchronized void initializeRegistry() throws RemoteException {

        if (this.shouldCreateRegistry) {
            Registry.getOrCreateRegistry(this.registryPortNumber);
            this.registryChecked = true;
        }
        if (!this.registryChecked) {
            Registry.getOrCreateRegistry(this.registryPortNumber);
            this.registryChecked = true;
        }
    }

    /**
     * Get the registry.
     *
     * @return
     *     the registry
     */
    public static java.rmi.registry.Registry getRegistry() {
        return Registry.registry;
    }

    /**
     * Get the registry creator.
     * @return
     *     the registry creator
     */
    public static boolean getRegistryCreator() {
        return Registry.registryCreator;
    }

    /**
     * Create the registry.
     *
     * @param port
     *          the port
     * @return
     *          the registry
     * @throws java.rmi.RemoteException
     *          impossible to realize a remote acces
     */
    private static java.rmi.registry.Registry createRegistry(final int port)
    throws java.rmi.RemoteException {
        Registry.registry = java.rmi.registry.LocateRegistry.createRegistry(port);
        Registry.registryCreator = true;
        return Registry.registry;
    }

    /**
     * Detect the registry.
     *
     * @param port
     *      the port
     * @return
     *     the registry
     */
    private static java.rmi.registry.Registry detectRegistry(
            final int port) {
        java.rmi.registry.Registry res = null;
        try {
            Registry.registry = java.rmi.registry.LocateRegistry.getRegistry(port);
            res = Registry.registry;

            if (res != null) {
                // call the registry
                Registry.registry.lookup("ping");
                res = Registry.registry;
            }
        } catch (java.rmi.NotBoundException e) {
            // registry found but not the object "ping"
            res = Registry.registry;

        } catch (java.rmi.RemoteException e) {
            res = null;
        }
        return res;
    }

    /**
     * Get or create the registry.
     *
     * @param port
     *       the port
     * @return
     *       the registry
     * @throws java.rmi.RemoteException
     *       impossible to realize a remote access
     */
    private static java.rmi.registry.Registry getOrCreateRegistry(
            final int port)
    throws java.rmi.RemoteException {
        java.rmi.registry.Registry res = null;
        // test if registry is already on
        Registry.registry = Registry.detectRegistry(port);
        res = Registry.registry;

        if (res == null) {
            // no registry created
            try {
                Registry.registry = Registry.createRegistry(port);
                res = Registry.registry;
            } catch (java.rmi.RemoteException e) {
                Registry.registry = Registry.detectRegistry(port);
                if (Registry.registry != null) {
                    res = Registry.registry;
                }
                System.err
                .println("Cannot detect an existing RMI Registry on port "
                        + port + " nor create one e=" + e);
                throw e;
            }
        }
        return res;
    }

    /**
     * Method registerRMIInterface.
     * @param comp
     *          the component context
     * @throws RemoteException
     *          impossible to register this object
     * @throws MalformedURLException
     *          impossible to register this object
     * @throws AlreadyBoundException
     *          impossible to register this object
     */
    public void registerRMIInterface(
            final RemoteComponentContext comp) throws RemoteException,
            MalformedURLException, AlreadyBoundException {
        Registry.registry.bind(this.getRmiComponentContextName(), comp);
    }



    /**
     * Method deregisterRMIInterface.
     * @throws RemoteException
     * @throws RemoteException
     *          impossible to register this object
     * @throws MalformedURLException
     *          impossible to register this object
     * @throws NotBoundException
     *          impossible to register this object
     */
    public void deregisterRMIInterface() throws RemoteException,
    MalformedURLException, NotBoundException {
        Registry.registry.unbind(this.getRmiComponentContextName());
    }
}
